@@ -96,6 +96,188 @@ void do_warn(SV* h, int rc, char* what) {
 }
 
 
+
+
+/***************************************************************************
+ *
+ *  Name:    _MyLogin, MyConnect
+ *
+ *  Purpose: Replacements for mysql_connect or msqlConnect
+ *
+ *  Input:   imp_dbh - database handle
+ *
+ *  Returns: TRUE for success, FALSE otherwise; you have to call
+ *           do_error in the latter case.
+ *
+ *  Bugs:    The msql version needs to set the environment
+ *           variable MSQL_TCP_PORT. There's absolutely no
+ *           portable way of setting environment variables
+ *           from within C: Neither setenv() nor putenv()
+ *           are guaranteed to work. I have decided to use
+ *           the internal perl functions setenv_getix()
+ *           and my_setenv() instead, let's hope, this is safe.
+ *
+ *           Another problem was pointed out by Andreas:
+ *           This isn't thread safe. We'll have fun with perl
+ *           5.005 ... :-)
+ *
+ **************************************************************************/
+
+int MyConnect(dbh_t *sock, char* host, char* port, char* user,
+	      char* password) {
+    int portNr;
+
+    if (host && !*host) host = NULL;
+    if (port && *port) {
+        portNr = atoi(port);
+    } else {
+        portNr = 0;
+    }
+    if (user && !*user) user = NULL;
+    if (password && !*password) password = NULL;
+
+    if (dbis->debug >= 2)
+        fprintf(DBILOGFP,
+		"imp_dbh->MyConnect: host = %s, port = %d, uid = %s," \
+                " pwd = %s\n",
+		host ? host : "NULL", portNr,
+		user ? user : "NULL",
+		password ? password : "NULL");
+
+#ifdef DBD_MYSQL
+    {
+#ifndef HAVE_MYSQL_REAL_CONNECT
+        /*
+	 *  Setting a port for mysql's client is ugly: We have to use
+	 *  the not documented variable mysql_port.
+	 */
+        mysql_port = portNr;
+        return mysql_connect(*sock, host, user, password) ? TRUE : FALSE;
+#else
+#if defined(MYSQL_VERSION_ID)  &&  MYSQL_VERSION_ID >= 032115
+	return mysql_real_connect(*sock, host, user, password, portNr, NULL,
+				  0) ?
+	    TRUE : FALSE;
+#else
+	return mysql_real_connect(*sock, host, user, password, portNr, NULL) ?
+	    TRUE : FALSE;
+#endif
+#endif
+    }
+#else
+    {
+        /*
+	 *  Setting a port for msql's client is extremely ugly: We have
+	 *  to set an environment variable. Even worse, we cannot trust
+	 *  in setenv or putenv being present, thus we need to use
+	 *  internal, not documented, perl functions. :-(
+	 */
+        char buffer[32];
+	char* oldPort = NULL;
+
+	sprintf(buffer, "%d", portNr);
+	if (portNr) {
+	    oldPort = environ[setenv_getix("MSQL_TCP_PORT")];
+	    if (oldPort) {
+	        char* copy = (char*) malloc(strlen(oldPort)+1);
+		if (!copy) {
+		    return FALSE;
+		}
+		strcpy(copy, oldPort);
+		oldPort = copy;
+	    }
+	    my_setenv("MSQL_TCP_PORT", buffer);
+	}
+	*sock = msqlConnect(host);
+	if (oldPort) {
+	    my_setenv("MSQL_TCP_PORT", oldPort);
+	    if (oldPort) { free(oldPort); }
+	}
+	return (*sock == -1) ? FALSE : TRUE;
+    }
+#endif
+}
+
+static int _MyLogin(imp_dbh_t *imp_dbh) {
+    SV* sv;
+    SV** svp;
+    HV* hv;
+    char* dbname;
+    char* host;
+    char* port;
+    char* user;
+    char* password;
+    STRLEN len;
+
+    sv = DBIc_IMP_DATA(imp_dbh);
+    if (!sv  ||  !SvROK(sv)) {
+        return FALSE;
+    }
+    hv = (HV*) SvRV(sv);
+    if (SvTYPE(hv) != SVt_PVHV) {
+        return FALSE;
+    }
+    if ((svp = hv_fetch(hv, "host", 4, FALSE))) {
+        host = SvPV(*svp, len);
+	if (!len) {
+	    host = NULL;
+	}
+    } else {
+        host = NULL;
+    }
+    if ((svp = hv_fetch(hv, "port", 4, FALSE))) {
+        port = SvPV(*svp, na);
+    } else {
+        port = NULL;
+    }
+    if ((svp = hv_fetch(hv, "user", 4, FALSE))) {
+        user = SvPV(*svp, len);
+	if (!len) {
+	    user = NULL;
+	}
+    } else {
+        user = NULL;
+    }
+    if ((svp = hv_fetch(hv, "password", 8, FALSE))) {
+        password = SvPV(*svp, len);
+	if (!len) {
+	    user = NULL;
+	}
+    } else {
+        password = NULL;
+    }
+    if ((svp = hv_fetch(hv, "database", 8, FALSE))) {
+        dbname = SvPV(*svp, na);
+    } else {
+        dbname = NULL;
+    }
+
+    if (dbis->debug >= 2)
+        fprintf(DBILOGFP,
+		"imp_dbh->MyLogin: dbname = %s, uid = %s, pwd = %s," \
+		"host = %s, port = %s\n",
+		dbname ? dbname : "NULL",
+		user ? user : "NULL",
+		password ? password : "NULL",
+		host ? host : "NULL",
+		port ? port : "NULL");
+
+#ifdef DBD_MYSQL
+    imp_dbh->svsock = &imp_dbh->mysql;
+#endif
+
+    if (!MyConnect(&imp_dbh->svsock, host, port, user, password)) {
+        return FALSE;
+    }
+
+    if (MySelectDb(imp_dbh->svsock, dbname)) {
+        MyClose(imp_dbh->svsock);
+	return FALSE;
+    }
+    return TRUE;
+}
+
+
 /***************************************************************************
  *
  *  Name:    dbd_db_login
@@ -122,12 +304,7 @@ int dbd_db_login(SV* dbh, imp_dbh_t* imp_dbh, char* dbname, char* user,
 	       user ? user : "NULL",
 	       password ? password : "NULL");
 
-#xtract Msql
-    if (!MyLogin(&imp_dbh->svsock, dbname, user, password)) {
-#xtract Mysql
-    imp_dbh->svsock = &imp_dbh->mysql;
-    if (!MyLogin(imp_dbh->svsock, dbname, user, password)) {
-#endxtract
+    if (!_MyLogin(imp_dbh)) {
 	DO_ERROR(dbh, MyErrno(imp_dbh->svsock, JW_ERR_CONNECT),
 		 imp_dbh->svsock);
 	return FALSE;
@@ -136,7 +313,7 @@ int dbd_db_login(SV* dbh, imp_dbh_t* imp_dbh, char* dbname, char* user,
     /*
      *  Tell DBI, that dbh->disconnect should be called for this handle
      */
-    DBIc_on(imp_dbh, DBIcf_ACTIVE);
+    DBIc_ACTIVE_on(imp_dbh);
 
     /*
      *  Tell DBI, that dbh->destroy should be called for this handle
@@ -195,7 +372,7 @@ int dbd_db_rollback(SV* dbh, imp_dbh_t* imp_dbh) {
 int dbd_db_disconnect(SV* dbh, imp_dbh_t* imp_dbh) {
     /* We assume that disconnect will always work       */
     /* since most errors imply already disconnected.    */
-    DBIc_off(imp_dbh, DBIcf_ACTIVE);
+    DBIc_ACTIVE_off(imp_dbh);
     if (dbis->debug >= 2)
         fprintf(DBILOGFP, "imp_dbh->svsock: %lx\n", (long) &imp_dbh->svsock);
     MyClose(imp_dbh->svsock );
@@ -327,22 +504,78 @@ SV* dbd_db_FETCH_attrib(SV* dbh, imp_dbh_t* imp_dbh, SV* keysv) {
     STRLEN kl;
     char *key = SvPV(keysv, kl);
 
-    if (kl==10 && strEQ(key, "AutoCommit")){
-        /*
-	 *  We do support neither transactions nor "AutoCommit".
-	 *  But we stub it. :-)
-	 */
-        return &sv_yes;
-    }
-    if (kl == 5  &&  strEQ(key, "errno")) {
+    switch (*key) {
+      case 'A':
+	if (strEQ(key, "AutoCommit")){
+	    /*
+	     *  We do support neither transactions nor "AutoCommit".
+	     *  But we stub it. :-)
+	     */
+	    return &sv_yes;
+	}
+	break;
+      case 'e':
+	if (strEQ(key, "errno")) {
 #if defined(DBD_MYSQL)  &&  defined(mysql_errno)
-	return sv_2mortal(newSViv((IV)mysql_errno(imp_dbh->svsock)));
+	    return sv_2mortal(newSViv((IV)mysql_errno(imp_dbh->svsock)));
 #else
-	return sv_2mortal(newSViv(-1));
+	    return sv_2mortal(newSViv(-1));
+#endif
+	} else if (strEQ(key, "errmsg")) {
+	    char* msg = MyError(imp_dbh->svsock);
+	    return sv_2mortal(newSVpv(msg, strlen(msg)));
+	}
+	break;
+      case 'h':
+	if (strEQ(key, "hostinfo")) {
+	    char* hostinfo = MyGetHostInfo(imp_dbh->svsock);
+	    return hostinfo ?
+	        sv_2mortal(newSVpv(hostinfo, strlen(hostinfo))) : &sv_undef;
+	}
+	break;
+#if defined(DBD_MYSQL)
+      case 'i':
+	if (strEQ(key, "info")) {
+	    char* info = mysql_info(imp_dbh->svsock);
+	    return info ? sv_2mortal(newSVpv(info, strlen(info))) : &sv_undef;
+	}
+	break;
+#endif
+      case 'p':
+	if (kl == 9  &&  strEQ(key, "protoinfo")) {
+	    char* protoinfo = MyGetProtoInfo(imp_dbh->svsock);
+	    return protoinfo ?
+	        sv_2mortal(newSVpv(protoinfo, strlen(protoinfo))) : &sv_undef;
+	}
+	break;
+      case 's':
+	if (kl == 10  &&  strEQ(key, "serverinfo")) {
+	    char* serverinfo = MyGetServerInfo(imp_dbh->svsock);
+	    return serverinfo ?
+	      sv_2mortal(newSVpv(serverinfo, strlen(serverinfo))) : &sv_undef;
+	} else if (strEQ(key, "sock")) {
+	    return sv_2mortal(newSViv((IV) imp_dbh->svsock));
+#if defined DBD_MYSQL
+	} else if (strEQ(key, "sockfd")) {
+	    return sv_2mortal(newSViv((IV) imp_dbh->svsock->net.fd));
+#endif
+	} else if (strEQ(key, "stats")) {
+#if defined(DBD_MYSQL)
+	    char* stats = mysql_stat(imp_dbh->svsock);
+	    return stats ?
+	        sv_2mortal(newSVpv(stats, strlen(stats))) : &sv_undef;
+#elif defined(DBD_MSQL)
+	    return sv_2mortal(newSViv(msqlGetServerStats(imp_dbh->svsock)));
+#endif
+	}
+	break;
+#if defined(DBD_MYSQL)
+      case 't':
+	if (kl == 9  &&  strEQ(key, "thread_id")) {
+	    return sv_2mortal(newSViv(mysql_thread_id(imp_dbh->svsock)));
+	}
+	break;
 #endif
-    } else if (kl == 6  &&  strEQ(key, "errmsg")) {
-	char* msg = MyError(imp_dbh->svsock);
-	return sv_2mortal(newSVpv(msg, strlen(msg)));
     }
     return Nullsv;
 }
@@ -380,14 +613,14 @@ int dbd_st_prepare(SV* sth, imp_sth_t* imp_sth, char* statement, SV* attribs) {
     imp_sth->done_desc = 0;
     imp_sth->cda = NULL;
     imp_sth->currow = 0;
-#xtract Mysql
+#if defined(DBD_MYSQL)
     imp_sth->use_mysql_use_result = 0;
-#endxtract
+#endif
     for (i = 0;  i < AV_ATTRIB_LAST;  i++) {
 	imp_sth->av_attr[i] = Nullav;
     }
 
-#xtract Mysql
+#if defined(DBD_MYSQL)
     /*
      *  Check attributes
      */
@@ -400,7 +633,7 @@ int dbd_st_prepare(SV* sth, imp_sth_t* imp_sth, char* statement, SV* attribs) {
 	    }
 	}
     }
-#endxtract
+#endif
 
     /*
      *  Allocate memory for parameters
@@ -485,7 +718,7 @@ int dbd_st_internal_execute(SV* h, SV* statement, SV* attribs, int numParams,
 	    }
 
 	    return 0;
-#xtract Msql
+#if defined(DBD_MSQL)
 	} else if (tolower(sbuf[4]) == 'i'
 		   &&  tolower(sbuf[5]) == 'n'
 		   &&  tolower(sbuf[6]) == 'd'
@@ -533,7 +766,7 @@ int dbd_st_internal_execute(SV* h, SV* statement, SV* attribs, int numParams,
 	    }
 
 	    return 0;
-#endxtract
+#endif
 	}
     }
 
@@ -547,14 +780,14 @@ int dbd_st_internal_execute(SV* h, SV* statement, SV* attribs, int numParams,
     Safefree(salloc);
 
     /** Store the result from the Query */
-#xtract Mysql
+#if defined(DBD_MYSQL)
     if (!(*cdaPtr = (use_mysql_use_result ?
 		     mysql_use_result(svsock) : mysql_store_result(svsock)))) {
         return mysql_affected_rows(svsock);
-#xtract Msql
+#elif defined(DBD_MSQL)
     if (!(*cdaPtr = MyStoreResult(svsock))) {
         return -1;
-#endxtract
+#endif
     }
 
     return MyNumRows((*cdaPtr));
@@ -609,9 +842,9 @@ int dbd_st_execute(SV* sth, imp_sth_t* imp_sth) {
 				     imp_sth->use_mysql_use_result))
 	!= -2) {
 	if (!imp_sth->cda) {
-#xtract Mysql
+#if defined(DBD_MYSQL)
 	    imp_sth->insertid = mysql_insert_id(imp_dbh->svsock);
-#endxtract
+#endif
 	} else {
 	    /** Store the result in the current statement handle */
 	    DBIc_ACTIVE_on(imp_sth);
@@ -671,9 +904,9 @@ AV* dbd_st_fetch(SV* sth, imp_sth_t* imp_sth) {
     int i;
     AV *av;
     row_t cols;
-#xtract Mysql
+#if defined(DBD_MYSQL)
     unsigned int* lengths;
-#endxtract
+#endif
 
     ChopBlanks = DBIc_is(imp_sth, DBIcf_ChopBlanks);
     if (dbis->debug >= 2) {
@@ -687,18 +920,20 @@ AV* dbd_st_fetch(SV* sth, imp_sth_t* imp_sth) {
 
     imp_sth->currow++;
     if (!(cols = MyFetchRow(imp_sth->cda))) {
-#xtract Mysql
+#if defined(DBD_MYSQL)
         if (!mysql_eof(imp_sth->cda)) {
 	    D_imp_dbh_from_sth;
 	    DO_ERROR(sth, JW_ERR_FETCH_ROW, imp_dbh->svsock);
 	}
-#endxtract
-        dbd_st_finish(sth, imp_sth);
+#endif
+	if (!DBIc_COMPAT(imp_sth)) {
+	    dbd_st_finish(sth, imp_sth);
+	}
 	return Nullav;
     }
-#xtract Mysql
+#if defined(DBD_MYSQL)
     lengths = mysql_fetch_lengths(imp_sth->cda);
-#endxtract
+#endif
     av = DBIS->get_fbav(imp_sth);
     num_fields = AvFILL(av)+1;
 
@@ -707,11 +942,11 @@ AV* dbd_st_fetch(SV* sth, imp_sth_t* imp_sth) {
 	SV *sv = AvARRAY(av)[i]; /* Note: we (re)use the SV in the AV	*/
 
 	if (col) {
-#xtract Mysql
+#if defined(DBD_MYSQL)
 	    STRLEN len = lengths[i];
-#xtract Msql
+#elif defined(DBD_MSQL)
 	    STRLEN len = strlen(col);
-#endxtract
+#endif
 	    if (ChopBlanks) {
 		while(len && isspace(*col)) {
 		    ++col;
@@ -832,11 +1067,11 @@ int dbd_st_STORE_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv, SV* valuesv) {
 		(u_long) sth, key);
     }
 
-#xtract Mysql
+#if defined(DBD_MYSQL)
     if (strEQ(key, "mysql_use_result")) {
         imp_sth->use_mysql_use_result = SvTRUE(valuesv);
     }
-#endxtract
+#endif
 
     if (dbis->debug >= 2) {
         fprintf(DBILOGFP,
@@ -875,15 +1110,15 @@ int dbd_st_STORE_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv, SV* valuesv) {
 #define IS_KEY(A) (((A) & (PRI_KEY_FLAG | UNIQUE_KEY_FLAG | MULTIPLE_KEY_FLAG)) != 0)
 #endif
 #ifndef IS_NUM
-#xtract Mysql
+#if defined(DBD_MYSQL)
 #define IS_NUM(A) ((A) >= (int) FIELD_TYPE_DECIMAL && (A) <= FIELD_TYPE_DATETIME)
-#xtract Msql
+#elif defined(DBD_MSQL)
 #ifdef UINT_TYPE
 #define IS_NUM(A) ((A) == INT_TYPE || (A) == REAL_TYPE || (A) == UINT_TYPE)
 #else
 #define IS_NUM(A) ((A) == INT_TYPE || (A) == REAL_TYPE)
 #endif
-#endxtract
+#endif
 #endif
 
 SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
@@ -950,7 +1185,7 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
 			int id;
 			const char* name;
 		    } types [] = {
-#xtract Mysql
+#if defined(DBD_MYSQL)
 			{ FIELD_TYPE_BLOB, "blob" },
 			{ FIELD_TYPE_CHAR, "char" },
 			{ FIELD_TYPE_DECIMAL, "decimal" },
@@ -969,7 +1204,7 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
 			{ FIELD_TYPE_TIMESTAMP, "timestamp" },
 			{ FIELD_TYPE_TIME, "time" },
 			{ FIELD_TYPE_VAR_STRING, "varstring" }
-#xtract Msql
+#elif defined(DBD_MSQL)
 			{ INT_TYPE, "int" },
 			{ CHAR_TYPE, "char" },
 			{ REAL_TYPE, "real" },
@@ -995,7 +1230,7 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
 #ifdef SYSVAR_TYPE
 			{ SYSVAR_TYPE, "sys" }
 #endif
-#endxtract
+#endif
 		    };
 		    int i, found = FALSE;
 
@@ -1014,7 +1249,7 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
 		    }
 		}
 	        break;
-#xtract Mysql
+#if defined(DBD_MYSQL)
 	      case AV_ATTRIB_MAX_LENGTH:
 		sv = newSViv((int) curField->max_length);
 		break;
@@ -1024,7 +1259,7 @@ SV* dbd_st_FETCH_internal(SV* sth, int what, result_t res, int cacheit) {
 	      case AV_ATTRIB_IS_BLOB:
 		sv = boolSV(IS_BLOB(curField->flags));
 		break;
-#endxtract
+#endif
 	    }
 
 	    av_push(av, sv);
@@ -1076,6 +1311,14 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
     }
 
     switch (*key) {
+#ifdef DBD_MYSQL
+      case 'a':
+	if (strEQ(key, "affected_rows")) {
+	    D_imp_dbh_from_sth;
+	    retsv = sv_2mortal(newSViv((IV)mysql_affected_rows(imp_dbh->svsock)));
+	}
+	break;
+#endif
       case 'I':
 	/*
 	 *  Deprecated, use lower case versions.
@@ -1084,12 +1327,12 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_PRI_KEY);
 	} else if (strEQ(key, "IS_NOT_NULL")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_NOT_NULL);
-#xtract Mysql
+#if defined(DBD_MYSQL)
 	} else if (strEQ(key, "IS_KEY")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_KEY);
 	} else if (strEQ(key, "IS_BLOB")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_BLOB);
-#endxtract
+#endif
 	} else if (strEQ(key, "IS_NUM")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_NUM);
 	}
@@ -1102,7 +1345,7 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_LENGTH);
 	}
 	break;
-#xtract Mysql
+#if defined(DBD_MYSQL)
       case 'M':
 	/*
 	 *  Deprecated, use max_length
@@ -1111,7 +1354,7 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_MAX_LENGTH);
 	}
 	break;
-#endxtract
+#endif
       case 'N':
 	if (strEQ(key, "NAME")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_NAME);
@@ -1143,10 +1386,10 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
       case 'f':
 	if (strEQ(key, "format_max_size")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_LENGTH);
-#xtract Mysql
+#if defined(DBD_MYSQL)
 	} else if (strEQ(key, "format_default_size")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_MAX_LENGTH);
-#endxtract
+#endif
 	} else if (strEQ(key, "format_right_justify")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_NUM);
 	} else if (strEQ(key, "format_type_name")) {
@@ -1160,12 +1403,12 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_PRI_KEY);
 	} else if (strEQ(key, "is_not_null")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_NOT_NULL);
-#xtract Mysql
+#if defined(DBD_MYSQL)
 	} else if (strEQ(key, "is_key")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_KEY);
 	} else if (strEQ(key, "is_blob")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_BLOB);
-#endxtract
+#endif
 	} else if (strEQ(key, "is_num")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_IS_NUM);
 	}
@@ -1175,7 +1418,7 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_LENGTH);
 	}
 	break;
-#xtract Mysql
+#if defined(DBD_MYSQL)
       case 'm':
 	if (strEQ(key, "max_length")) {
 	    retsv = ST_FETCH_AV(AV_ATTRIB_MAX_LENGTH);
@@ -1183,7 +1426,7 @@ SV* dbd_st_FETCH_attrib(SV* sth, imp_sth_t* imp_sth, SV* keysv) {
 	    retsv = boolSV(imp_sth->use_mysql_use_result);
 	}
 	break;
-#endxtract
+#endif
       case 'r':
 	if (strEQ(key, "result")) {
 	    retsv = sv_2mortal(newSViv((IV) imp_sth->cda));
@@ -1294,7 +1537,7 @@ int dbd_bind_ph (SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
 }
 
 
-#xtract Mysql
+#if defined(DBD_MYSQL)
 
 /***************************************************************************
  *
@@ -1311,12 +1554,6 @@ int dbd_bind_ph (SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
 int MysqlReconnect(SV* h) {
     D_imp_xxh(h);
     imp_dbh_t* imp_dbh;
-    SV* sv;
-    HV* hv;
-    SV** svp;
-    SV* dbname;
-    SV* user;
-    SV* password;
 
     if (DBIc_TYPE(imp_xxh) == DBIt_ST) {
         imp_dbh = (imp_dbh_t*) DBIc_PARENT_COM(imp_xxh);
@@ -1325,26 +1562,7 @@ int MysqlReconnect(SV* h) {
         imp_dbh = (imp_dbh_t*) imp_xxh;
     }
 
-    sv = DBIc_IMP_DATA(imp_dbh);
-    if (!sv  ||  !SvROK(sv)  ||  SvTYPE(SvRV(sv)) != SVt_PVHV) {
-        return FALSE;
-    }
-
-    hv = (HV*) SvRV(sv);
-    if (!(svp = hv_fetch(hv, "dsn", 3, FALSE))) {
-        return FALSE;
-    }
-    dbname = *svp;
-    if (!(svp = hv_fetch(hv, "user", 4, FALSE))) {
-        return FALSE;
-    }
-    user = *svp;
-    if (!(svp = hv_fetch(hv, "password", 8, FALSE))) {
-        return FALSE;
-    }
-    password = *svp;
-    if (!MyLogin(imp_dbh->svsock, SvPV(dbname,na), SvPV(user,na),
-		 SvPV(password,na))) {
+    if (!_MyLogin(imp_dbh)) {
 	DO_ERROR(h, MyErrno(imp_dbh->svsock, JW_ERR_CONNECT),
 		 imp_dbh->svsock);
 	return FALSE;
@@ -1352,4 +1570,4 @@ int MysqlReconnect(SV* h) {
     return TRUE;
 }
 
-#endxtract
+#endif
